#
#
#
require(e1071)
#
# The state evolution LUT for binarization noise removal
# (each pixel state is represented with 1 bit): a pixel is
# set to 1 if at least half the number of pixels in the
# neighborhood is at 1
#
ia.camNoiseRemovalLut <-  function(neighborhood = 4) {
  #
  # the number of bits required to represent
  # the local state
  #
  nb <- neighborhood+1
  #
  # and its half
  #
  n  <- nb / 2L
  #
  # generate the bit breakdown of all (integer)
  # packed state information
  #
  bc <- bincombinations(nb)
  N  <- 2**(nb)
  #
  # prepare the look up table (lut) for storing
  # state transitions 
  #
  lut <- array(0L, dim=c(N))
  #
  # and fill it according to the principles
  # outlined above
  #
  for(i in 1:N) {
    if(sum(bc[i,]) > n)
      lut[i] <- 1L
    else
      lut[i] <- 0L
  }
  #
  # returning it as a labeled list
  #
  list(lutOffset=0L,
       lutData=lut)
  #
}

ia.cam <- function(img,  stateBits, stateLut, neighborhood = 4, steps = 1) {
  #
  # CAM are characterized by discrete states, time and space
  #
  if(img@storage != "integer")
    stop("integer animage required")
  #
  # prevent history cluttering
  #
  document     <- img@document
  img@document <- FALSE
  #
  # determine required image translations based on
  # selected neighborhood
  #
  if(neighborhood == 4) {
    nn <- list(c(0,0),
               c(-1,0), c(1,0), c(0,1),c(0,-1))
    ss <- (0:4)*stateBits
  } else {
    nn <- list(c(0,0),
               c(-1,0), c(-1,1), c(0,1),c(1,1),
               c(1,0), c(1,-1), c(0,-1),c(-1,-1))
    ss <- (0:8)*stateBits
  }
  #
  # the elementary function translating the
  # image and packing states into a single value
  #
  f   <- function(a, b) {
    ia.shift(ia.slide(res, -a[1], -a[2]), b)
  }
  #
  # the actual evolution loop
  #
  res <- img
  for(t in 1:steps) {
    state <- ia.reduce(ia.add,
                       ia.map(f, nn, ss))
    res   <- ia.table(state, stateLut$lutData, stateLut$lutOffset)
    res@outside <- img@outside
  }
  #
  # restore history
  #
  img@document <- document
  #
  # and update it in the result
  # 
  if(is.call(updatedHistory <- ia.updatedHistory(match.call()))) {
    res@history  <- updatedHistory
    res@document <- TRUE
  } else {
    res@history  <- call("<undef>")
    res@document <- FALSE
  }
  #
  res
  #
}
