\name{aniMorphology}

\alias{ia.mmBitDilation}
\alias{ia.mmBitErosion}
\alias{ia.mmBitOpening}
\alias{ia.mmBitClosing}
\alias{ia.mmGreyDilation}
\alias{ia.mmGreyErosion}
\alias{ia.mmGreyOpening}
\alias{ia.mmGreyClosing}
\alias{ia.mmTopHatTransform}

\title{Mathematical morphology}

\description{

  An implementation of the basic mathematical morphology operators for
  binary and grey level images.
  
}

\usage{
ia.mmBitDilation(img, strctElt)
ia.mmBitErosion(img, strctElt)
ia.mmBitOpening(img, strctElt)
ia.mmBitClosing(img, strctElt)

ia.mmGreyDilation(img, strctElt)
ia.mmGreyErosion(img, strctElt)
ia.mmGreyOpening(img, strctElt)
ia.mmGreyClosing(img, strctElt)

ia.mmTopHatTransform(img, strctElt, mode="white")
}

\arguments{
    \item{img}{an integer \code{animage}}
    \item{strctElt}{an integer \code{animage}}
    \item{mode}{"white" or "black"}
}

\value{
 An integer \code{animage}.
}

\details{
  
  Mathematical morphology relates to the processing of geometrical
  structures. It is most commonly applied to binary digital images,
  considered as sets, and it is characterized by a large number of
  operators that can be used to extract quantitative image descriptors
  and to transform images. The theory can be generalized to grey level
  images (and even to much more general spatial structures).

  AnImAl provides the basic operators for binary and grey level images.
  Images are analyzed by means of a structuring element, an image
  itself, computing how it fits image structure. The structuing element
  of binary images should be a binary image itself, while for grey level
  images it is a grey level image.

  Dilation (erosion) is closely related to Minkowski addition (subtraction)
  which, for two sets A and be in Euclidean space, is obtained by adding
  all elements of B (the structuring element) to all elements of A.
  Opening is defined as the erosion of A by B followed by a dilation by
  B, while closing is defined as the dilation of A by B, followed by
  erosion by B. Opening and closing are idempotent: the former is
  anti-extensive, the latter is extensive.

  The top-hat  transform is an  operation built on top of the previous
  operator and it extracts small  elements and details from given images
  (where small means at the scale of the structuring element).

  The white top-hat  transform is defined as the  difference between the
  input image and its opening, the black top-hat transform is defined as
  the difference between the closing and the input image.
  
  }

\author{Roberto Brunelli \email{brunelli@fbk.eu}}


\references{
  Robert M. Haralick (2008).
  \emph{Mathematical Morphology: Theory and Hardware},
  Oxford Series in Optical and Imaging Sciences, Oxford University Press
}

\examples{
#
oldpar        <- par(no.readonly = TRUE)
#
par(mfrow   = c(2,2),
    mar     = c(3,0,0,0),
    mgp     = c(1,1,1),
    oma     = c(0,0,3,0),
    cex.lab = 1.5)
#
# get paths to package samples
#
sampleimages <- file.path(system.file(package = "AnImAl"), "sampleimages/")
#
# Read a sample images
#
imgA         <- ia.integerAnimage(
                  as.animage(
                   getChannels(
                    read.pnm(file.path(sampleimages,"sampleFace_07.pgm")))*255))
#
# A romboidal structuring element
#
eltA <- animage(array(as.integer(c(0,0,1,0,0,
                                   0,1,1,1,0,
                                   1,1,1,1,1,
                                   0,1,1,1,0,
                                   0,0,1,0,0)),
                      c(5, 5)))
#
# Create a paraboloid as grey level structuring
# element
#
eltB <- animage(array(0, c(5,5)), focus = c(-2L,-2L))
#
for(j in -2:2) {
  for(i in -2:2) {
    eltB[j,i] = 6*(8 - j*j -i*i)
  }
}
#
imgAalt <- ia.greater(imgA, 155L)
for(i in 1:3) imgAalt <- ia.mmBitDilation(imgAalt, eltA)
#
ia.show(imgAalt, xlab="Dilation (x 3)")
#
for(i in 1:3)
 imgAalt <- ia.mmBitErosion(imgAalt, eltA)
#
ia.show(imgAalt, xlab="Erosion (x 3)")
#
ia.show(
  ia.not(ia.frame(
   ia.mmTopHatTransform(imgA, eltB,mode="white"),2L)),
 xlab="Top-hat ('white' variant, inverted result)")
#
ia.show(
  ia.not(ia.frame(
   ia.mmTopHatTransform(imgA, eltB,mode="black"),2L)),
 xlab="Top-hat ('black' variant, inverted result)")
#
par(oldpar)
#
}
\keyword{array}
